function UpdateAnAgent(k)
% Updates agent k. This includes:
%       a. Updating clock of the agent
%       b. If at a communication timestep (i.e. 'Time(k)' is multiple of 'tau'),
%          then communicate with neighbours and update its A and R matrices
%       c. If at a velocity command issue timestep (i.e. 'Time(k)' is a multiple of 'delta_t'),
%          then issue new velocity commands

global SimOptions
global G
global botNos x xd xdCG xd_local
global Agent_CommunicationInterval Agent_CommandInterval
global GlobalTimeStep
global CriticalRadius maxVelMagnitude
global Agent
global SafetyFactor_CriticalRadius

% ----------------------------------------------------

% Update the cock
Agent(k).t = Agent(k).t + GlobalTimeStep;

% ----------------------------------------------------

% Communicate with neighbors
% Assumption: When it communicates, it also measures the distances from neighbors using the range sensor
if Agent(k).t_communication + Agent_CommunicationInterval <= Agent(k).t

    % Identify the neighbors
    NeighborIndices = [];
    for q=1:G.NodesN
        if G.A(k,q)
            NeighborIndices = [NeighborIndices, q];
        end
    end

    % Update (a,b)-th element of each matrix
    for a=1:G.NodesN,
        for b=1:G.NodesN,

            if ((a==k) || (b==k)) && (a~=b)

                OtherAgent = setdiff([a b], k);
                Agent(k).E(a,b) = 1;
                if any(NeighborIndices==OtherAgent)
                    Agent(k).A(a,b) = 1;
                    Agent(k).R(a,b) = norm(x(k,:)-x(OtherAgent,:));
                else
                    Agent(k).A(a,b) = 0;
                    Agent(k).R(a,b) = Inf;
                end

            else

                NeighborInformationAge = [];
                for q=NeighborIndices
                    NeighborInformationAge = [NeighborInformationAge, Agent(q).E(a,b)];
                end
                [MostRecentAge, MostRecentIndex] = min(NeighborInformationAge);
                MostRecentNeighbor = NeighborIndices(MostRecentIndex);
                Agent(k).E(a,b) = MostRecentAge + 1;
                Agent(k).A(a,b) = Agent(MostRecentNeighbor).A(a,b);
                Agent(k).R(a,b) = Agent(MostRecentNeighbor).R(a,b);

            end

        end
    end
    
    Agent(k).t_communication = Agent(k).t;
end

% ----------------------------------------------------

% Decouple the local and global components of the velocities
if SimOptions.DoVelocityDecoupling
    xCG = mean(x);
    x_local = x - repmat(xCG, [botNos 1]);
else
    x_local = x;
end

% Computing and setting velocity commands
% Assumption: When it computes velocity, it also measures the distances from neighbors using the range sensor
if Agent(k).t_command + Agent_CommandInterval <= Agent(k).t
    
    % Get the velocity command from primary controller
    Agent(k).v = 5 * (xd_local(k,:) - x_local(k,:));
    Magni = norm(Agent(k).v);
    if Magni > maxVelMagnitude
        Agent(k).v = maxVelMagnitude * Agent(k).v / Magni;
    end
    
    % Identify the neighbors
    NeighborIndices = [];
    for q=1:G.NodesN
        if G.A(k,q)
            NeighborIndices = [NeighborIndices, q];
        end
    end
    
    % Detect the critical edges
    critNeigh = detectCriticalEdges(Agent(k), k);
    critNeigh = intersect(NeighborIndices, critNeigh); % Just to make sure that these are neighbors
    
    % modify the velocity
    scaleFactor = 1;
    scaleMat = eye(2);
    Vel = (Agent(k).v)';
    for b=critNeigh,
        thisScaleMat = modifyVelocity_1(Vel, (x(b,:)-x(k,:))', SafetyFactor_CriticalRadius*CriticalRadius);
        scaleMat = thisScaleMat * scaleMat;
    end
    newVel = scaleMat * Vel;
    for a=1:length(critNeigh),
        b = critNeigh(a);
        scaleFactor(a+1) = modifyVelocity_2(newVel, (x(b,:)-x(k,:))', ...
            SafetyFactor_CriticalRadius*CriticalRadius, Agent_CommandInterval);
    end
    Agent(k).v = (min(scaleFactor)* newVel)';
    
    % Adding the global component
    if SimOptions.DoVelocityDecoupling
        Agent(k).v = Agent(k).v + 2*(xdCG-xCG);
    end
    
end
