function neibIndx = detectCriticalEdges(AgentInstance, i)
% Detects the 'critical' edges in the graph defined by Adjuscency matrix
% 'A' only for the nodes connecting the agent 'i'.
global maxVelMagnitude CriticalRadius
global Agent_CommunicationInterval Agent_CommandInterval
global SafetyFactor_SubcriticalEdgeLength

Emat = AgentInstance.E;
Ad = AgentInstance.A;
Ra = AgentInstance.R;

nodesNo = size(Ad,1);

% Eliminating the nodes which are not connected to i
subAindx = [];
for a=1:nodesNo,
    if sum(Ad(a,:)) ~= 0
        subAindx = [subAindx a];
        if a==i
            iSub = size(subAindx,2);
        end
    end
end

subA = Ad(subAindx,subAindx);
subR = Ra(subAindx,subAindx);
subNodesNo = size(subA,1);

% Identifying the sub-critical Agents
% This is computed simply based on the distances
subcriticalA = zeros(size(subA));
subcriticalEdgeCount = 0;
for a=1:subNodesNo,
    for b=a:subNodesNo,
        if (subR(a,b) + maxVelMagnitude*(2*Agent_CommandInterval+Emat(a,b)*Agent_CommunicationInterval) ...
                    > SafetyFactor_SubcriticalEdgeLength*CriticalRadius)  &&  (subA(a,b)==1)
            subcriticalA(a,b) = 1;
            subcriticalA(b,a) = 1;
            subcriticalEdgeCount = subcriticalEdgeCount + 1;
        end
    end
end

% Testing subcritical edges
neibIndx = [];
p = 0;
while any(subA(iSub,:) & subcriticalA(iSub,:)) && (p <= sum(sum(subcriticalA))/2)
    % Stop if none of the remaining neighbouring edges (i.e. those which have not already
    % been detected as critical) are subcritical
    p = p + 1;

    for a=1:subNodesNo,
        if subA(a,iSub) && subcriticalA(a,iSub) % Only if a is a neighbouring node and a-iSub is subcritical
            % Construct n groups of p subcritical edges, each group containing the edge a-iSub
            SubcriticalEdgeGroups = makeSubcriticalEdgeGroups(subcriticalA, [a iSub], p);
            n = length(SubcriticalEdgeGroups);
            for b=1:n
                if isJointCriticalEdges(subA, SubcriticalEdgeGroups{b})
                    neibIndx = [neibIndx subAindx(a)];
                    subcriticalA(a,iSub) = 0;
                    subcriticalA(iSub,a) = 0;
                end
            end
        end
        if ~any(subA(iSub,:) & subcriticalA(iSub,:))
            % Break if none of the remaining neighbouring edges (i.e. those which have not already
            % been detected as critical) are subcritical
            break;
        end
    end

end


% ===============================================

function isCritical = isJointCriticalEdges(Ad, Edges)
% Given the adjacency matrix Ad.
% Removing all the edges mentioned in 'Edges'
% Ckeck whether the removal makes the graph disconnected
subNodesNo = size(Ad, 1);

% Removing the edges
for a = 1:size(Edges, 1)
    Ad(Edges(a,1),Edges(a,2)) = 0;
    Ad(Edges(a,2),Edges(a,1)) = 0;
end

% Constructing the Laplacian
%{
tempL = zeros(subNodesNo,subNodesNo);
for p=1:subNodesNo,
for q=1:subNodesNo,
if p==q
tempL(p,q) = sum(tempSubA(p,:));
end
if tempSubA(p,q) == 1
tempL(p,q) = -1;
end
end
end
%}

%%eigenvals = sort(eig(tempL));
isCritical = 1;
testSubA = eye(size(Ad));
powSubA = testSubA;
for b=1:subNodesNo,
    if min(min(testSubA)) ~= 0
        isCritical = 0;
        break;
    end
    powSubA = powSubA*Ad;
    testSubA = testSubA + powSubA;
end


% ===============================================

function SubcriticalEdgeGroups = makeSubcriticalEdgeGroups(As, MandetoryEdges, groupSize)
% Given a subcritical edge matrix As, and some among the subcritical edges which are to be chosen mandetorily,
% It creates all possible groups of subcritical edges of size 'groupSize'
% and returns the groups in the cell array SubcriticalEdgeGroups

MandetoryEdgeCount = size(MandetoryEdges,1);
ChooseCount = groupSize - MandetoryEdgeCount;
NodeCount = size(As,1);

if ChooseCount < 0
    SubcriticalEdgeGroups = -1;
    return;
end

% Removing the mandetory edges from As since the mandetory edges will anyhow be included
for a=1:MandetoryEdgeCount
    if As(MandetoryEdges(a,1),MandetoryEdges(a,2)) && As(MandetoryEdges(a,2),MandetoryEdges(a,1))
        As(MandetoryEdges(a,1),MandetoryEdges(a,2)) = 0;
        As(MandetoryEdges(a,2),MandetoryEdges(a,1)) = 0;
    else
        SubcriticalEdgeGroups = -1;
        return;
    end
end

% Generating a list of the remaining subcritical edges
RemainingSubcriticalEdges = [];
for p=1:NodeCount
    for q=(p+1):NodeCount
        if As(p,q)
            RemainingSubcriticalEdges = [RemainingSubcriticalEdges; [p,q]];
        end
    end
end
RemainingSubcriticalEdgesCount = size(RemainingSubcriticalEdges,1);
if (RemainingSubcriticalEdgesCount < ChooseCount) || (ChooseCount == 0)
    SubcriticalEdgeGroups{1} = MandetoryEdges;
    return;
end

% Choosing from the remaining subcritical edges
C = nchoosek(1:RemainingSubcriticalEdgesCount, ChooseCount);
for a = 1:size(C,1)
    SubcriticalEdgeGroups{a} = [MandetoryEdges; RemainingSubcriticalEdges((C(a,:))',:)];
end
